local L = LibStub("AceLocale-3.0"):GetLocale("LZProfiler", false)

--[[ Creating AddOn ]]
LZProfiler = LibStub("AceAddon-3.0"):NewAddon("LZProfiler", "AceConsole-3.0", "AceEvent-3.0")

--[[ Creating variables for various functions ]]
local LZRevision = tonumber((string.gsub("$Revision: 99 $", "^.-(%d+).-$", "%1")))
local LZTooltip -- Hidden tooltip to get item info
local LZRoot -- Root node
local LZParsedStatus = {}

--[[ Key IDs for dungeons attunments ]]
local LZKeyAccess = {
	-- Normal dungeons
	shattered_halls = {
		"28395"
	},
	arcatraz = {
		"31084"
	},
	karazhan = {
		"24490"
	},
	black_temple = {
		"32649"
	},
	
	-- Heroic dungeons
	hellfire_citadel = {
		"30622",
		"30637"
	},
	coilfang_reservoir = {
		"30623"
	},
	auchindoun = {
		"30633"
	},
	caverns_of_time = {
		"30635"
	},
	tempest_keep = {
		"30634"
	},
	
	-- Old dungeons
	scarlet_monastery = {
		"7146"
	},
	dire_maul = {
		"18249",
	},
	scholomance = {
		"13704",
	},
	onyxia = {
		"16309"
	},
	ubrs = {
		"12344"
	},
	
	-- Shortcuts
	gnomeregan = {
		"6893"
	},
	maraudon = {
		"17191"
	},
	blackrock_depths = {
		"11000"
	},
	stratholme = {
		"12382"
	},
	
	-- Optional bosses
	gahzrilla = {
		"9240"
	},
	hakkar_avatar = {
		"10818"
	},
	nightbane = {
		"24140"
	},
	theldren = {
		"21986"
	},
	valthalak = {
		"22057"
	},
	
	-- Special items
	eternal_quintessence = {
		"22754"
	}
}

--[[ Spell names for tradeskills specialization ]]
local LZKeyTradeskillsSpecialization = {
	alchemy_elixir_master = L.professions.alchemy.elixir_master,
	alchemy_potion_master = L.professions.alchemy.potion_master,
	alchemy_transmutation_master = L.professions.alchemy.transmutation_master,
	
	blacksmithing_armorsmithing = L.professions.blacksmithing.armorsmithing,
	blacksmithing_master_axesmithing = L.professions.blacksmithing.master_axesmithing,
	blacksmithing_master_hammersmithing = L.professions.blacksmithing.master_hammersmithing,
	blacksmithing_master_swordsmithing = L.professions.blacksmithing.master_swordsmithing,
	blacksmithing_weaponsmithing = L.professions.blacksmithing.weaponsmithing,
	
	engineering_gnomish = L.professions.engineering.gnomish,
	engineering_goblin = L.professions.engineering.goblin,
	
	leatherworking_dragonscale = L.professions.leatherworking.dragonscale,
	leatherworking_elemental = L.professions.leatherworking.elemental,
	leatherworking_tribal = L.professions.leatherworking.tribal,
	
	tailoring_mooncloth = L.professions.tailoring.mooncloth,
	tailoring_shadoweave = L.professions.tailoring.shadoweave,
	tailoring_spellfire = L.professions.tailoring.spellfire,
}

--[[ Initializing variables ]]
function LZProfiler:ResetSavedVariables()
	LZProfile = {
		Items = {},
		Spells = {},
		Characters = {},
		Revision = LZRevision
	}
end

--[[ Addon code ]]
function LZProfiler:OnInitialize()
	if not LZProfile then
		LZProfiler:ResetSavedVariables()
	elseif LZProfile.Locale ~= GetLocale() then
		LZProfiler:ResetSavedVariables()
	elseif LZProfile.Revision ~= LZRevision then
		LZProfiler:ResetSavedVariables()
	end
	
	local realmName = GetRealmName()
	local characterName = UnitName("player")
	
	if not LZProfile.Characters[realmName] then
		LZProfile.Characters[realmName] = {}
	end
	if not LZProfile.Characters[realmName][characterName] then
		LZProfile.Characters[realmName][characterName] = {}
	end
	
	LZProfile.Locale = GetLocale()
	
	LZRoot = LZProfile.Characters[realmName][characterName]
	
	if not LZRoot.Dungeons then
		LZRoot.Dungeons = {}
	end
	if not LZRoot.Professions then
		LZRoot.Professions = {}
	end
	if not LZRoot.Specializations then
		LZRoot.Specializations = {}
	end
	
	--[[ Temporary warning ]]
	--self:Print(L.tempWarning)
	
	--[[ Creating tooltip ]]
	LZTooltip = CreateFrame("GameTooltip", "LZGameTooltip", UIParent, "GameTooltipTemplate")
	LZTooltip:SetOwner(UIParent,"ANCHOR_NONE")
	
	--[[ Registering chat commands ]]
	local options = {
		type = 'group',
		args = {
			status = {
				type = 'execute',
				name = L.status,
				desc = L.statusDescription,
				func = function()
					self:PrintParseStatus()
				end,
			},
			purgeTrades = {
				type = 'execute',
				name = L.purgeTradeskills,
				desc = L.purgeTradeskillsDescription,
				func = function()
					self:PurgeTradeskills()
				end,
			},
			purgeAll = {
				type = 'execute',
				name = L.purgeAll,
				desc = L.purgeAllDescription,
				func = function()
					self:PurgeAll()
				end,
			}
		}
	}
	LibStub("AceConfig-3.0"):RegisterOptionsTable("LZProfiler", options, {"lzp", "lzprofiler"})
end

function LZProfiler:OnEnable()
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("BANKFRAME_OPENED")
	
	self:RegisterEvent("TIME_PLAYED_MSG")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("PLAYER_XP_UPDATE")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("PLAYER_GUILD_UPDATE")
	
	_, LZRoot.RaceEn = UnitRace("player")
	_, LZRoot.ClassEn = UnitClass("player")
	LZRoot.FactionEn = UnitFactionGroup("player")
	LZRoot.SexId = UnitSex("player")
	LZRoot.Level = UnitLevel("player")
	self:UpdateXP()
	LZRoot.timestampUpdate = time()
	
	RequestTimePlayed()
	
	if HasKey() then
		self:ScanBag(KEYRING_CONTAINER)
	end
	
	self:ScanBag(0)
	self:ScanSpellBook()
end

function LZProfiler:PLAYER_GUILD_UPDATE()
	local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
	
	LZRoot.Guild = {
		Name = guildName,
		Title = guildRankName,
		Rank = guildRankIndex
	}
end

function LZProfiler:PLAYER_XP_UPDATE()
	self:UpdateXP()
end

function LZProfiler:UpdateXP()
	LZRoot.CurrentExperience = UnitXP("player")
	LZRoot.MaxExperience = UnitXPMax("player")
	LZRoot.RestedExperience = GetXPExhaustion()
end

function LZProfiler:TIME_PLAYED_MSG(_, arg1, arg2)
	LZRoot.TimePlayed = arg1
	LZRoot.TimeLevelPlayed = arg2
end

function LZProfiler:PLAYER_LEVEL_UP(arg1)
	LZRoot.Level = arg1
end

function LZProfiler:BAG_UPDATE()
	if arg1 ~= nil then
		self:ScanBag(arg1)
	end
end

function LZProfiler:RegisterItem(itemID, itemName, itemColor, itemTexture)
	if itemID == nil or itemName == nil or itemColor == nil or itemTexture == nil then
		return false
	end
	
	if LZProfile.Items[itemID] then
		return true
	end
	
	LZProfile.Items[itemID] = {
		RealName = itemName,
		Color = itemColor,
		Texture = itemTexture
	}
	
	return true
end

function LZProfiler:RegisterSpell(spellID, spellName, spellColor, spellTexture)
	if spellID == nil or spellName == nil or spellColor == nil or spellTexture == nil then
		return false
	end
	
	if LZProfile.Spells[spellID] then
		return true
	end
	
	LZProfile.Spells[spellID] = {
		RealName = spellName,
		Color = spellColor,
		Texture = spellTexture
	}
	
	return true
end

function LZProfiler:ScanBag(bagID) --Bags = Root Node
	local bagSlots = GetContainerNumSlots(bagID)
	
	if bagSlots < 1 then
		return
	end
	
	local slot
	local itemLink
	local dungeonId, keyTable, keyId
	for slot=1, bagSlots do
		itemLink = GetContainerItemLink(bagID, slot)
		
		if itemLink ~= nil then
			_, itemID, _ = self:ItemLinkDecode(itemLink)
			
			if itemID ~= nil then
				for dungeonId,keyTable in pairs(LZKeyAccess) do
					if not LZRoot.Dungeons[dungeonId] then
						for _,keyId in pairs(keyTable) do
							if itemID == keyId then
								LZRoot.Dungeons[dungeonId] = true
							end
						end
					end
				end
			end
		end
	end
end

function LZProfiler:ScanSpellBook()
	local spellName, spellRank
	local i=1, k, v
	
	repeat
		spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
		
		if spellName ~= nil then
			for k,v in pairs(LZKeyTradeskillsSpecialization) do
				if spellName == v then
					LZRoot.Specializations[k] = true
				end
			end
		end
		
		i = i + 1
	until not spellName
end

function LZProfiler:ParseTradeSkill()
	local tradeskillName = GetTradeSkillLine()
	local index, skillType, reagent
	
	if LZParsedStatus[tradeskillName] == nil then
		LZParsedStatus[tradeskillName] = false
		self:Print(format(L.reagentAnalyseFailed, tradeskillName))
	end
	
	for index=1, GetNumTradeSkills() do
		_, skillType, _, _ = GetTradeSkillInfo(index)
		
		if skillType ~= "header" then
			LZTooltip:SetTradeSkillItem(index)
			
			for reagent=1, GetTradeSkillNumReagents(index) do
				LZTooltip:SetTradeSkillItem(index, reagent)
			end
		end
	end
end

function LZProfiler:TRADE_SKILL_SHOW()
	if IsTradeSkillLinked() then
		return
	end
	
	local tradeskillName = GetTradeSkillLine()
	
	local T = {}
	
	local skillName, skillType
	local header = nil
	local itemLink, itemName, itemColor, itemID, itemTexture
	local spellColor, spellID, spellName, spellTexture
	local reagentName, reagentColor, reagentID, reagentTexture, reagentCount, reagentLink
	for index=1, GetNumTradeSkills() do
		skillName, skillType, _, _ = GetTradeSkillInfo(index)
		
		if skillType == "header" then
			header = skillName
			if not T[header] then
				T[header] = {}
			end
		else
			if header == nil then
				self:Print(L.tradeskillAnalyseFailed)
				return
			end
			
			itemLink = GetTradeSkillItemLink(index)
			
			if string.find(itemLink, "|Hitem:") then
				itemName, _, _, _, _, _, _, _, _, itemTexture = GetItemInfo(itemLink)
				itemColor, itemID, _ = self:ItemLinkDecode(itemLink)
				
				if not self:RegisterItem(itemID, itemName, itemColor, itemTexture) then
					self:ParseTradeSkill()
					return
				end
				
				T[header][skillName] = {
					Item = itemID,
					Reagents = {}
				}
			else
				spellColor, spellID, spellName = self:EnchantLinkDecode(itemLink)
				spellTexture = "Interface\\Icons\\Spell_Holy_GreaterHeal"
				
				if not self:RegisterSpell(spellID, spellName, spellColor, spellTexture) then
					self:ParseCraft()
					return
				end
				
				T[header][skillName] = {
					Spell = spellID,
					Reagents = {}
				}
			end
			
			for reagent=1, GetTradeSkillNumReagents(index) do
				reagentName, reagentTexture, reagentCount = GetTradeSkillReagentInfo(index, reagent)
				reagentLink = GetTradeSkillReagentItemLink(index, reagent)
				
				if reagentLink == nil then
					self:ParseTradeSkill()
					return
				end
				
				reagentColor, reagentID = self:ItemLinkDecode(reagentLink)
				
				if self:RegisterItem(reagentID, reagentName, reagentColor, reagentTexture) == false then
					self:ParseTradeSkill()
					return
				end
				
				T[header][skillName].Reagents[reagentID] = {
					Quantity = reagentCount
				}
			end
		end
	end
	
	if not LZRoot.Professions[tradeskillName] then
		LZRoot.Professions[tradeskillName] = {}
	end
	
	LZRoot.Professions[tradeskillName] = T
		
	if (LZParsedStatus[tradeskillName] == nil) or (LZParsedStatus[tradeskillName] == false) then
		self:Print(format(L.scanSuccessful, tradeskillName))
	end
	
	LZParsedStatus[tradeskillName] = true
end

function LZProfiler:BANKFRAME_OPENED()
	self:ScanBag(BANK_CONTAINER)
	for i=NUM_BAG_SLOTS+1, NUM_BAG_SLOTS+NUM_BANKBAGSLOTS do
		self:ScanBag(i)
	end
end

function LZProfiler:EnchantLinkDecode(enchantLink)
	local enchantColor, enchantID, enchantName
	for enchantColor, enchantID, enchantName in string.gmatch(enchantLink, "|c%x%x(%x+)|Henchant:(%d+)|h%[(.+)%]|h|r") do
		return enchantColor, enchantID, enchantName
	end
end

function LZProfiler:ItemLinkDecode(itemLink)
	local itemColor, itemID, itemName
	for itemColor, itemID, itemName in string.gmatch(itemLink, "|c%x%x(%x+)|Hitem:(%d+):%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+|h%[(.+)%]|h|r") do
		return itemColor, itemID, itemName
	end
end

function LZProfiler:PrintParseStatus()
	self:Print(L.tradeskillsScanned)
	
	local k, v, msg_status
	local has_tradeskill = false
	
	for k,v in pairs(LZParsedStatus) do
		if has_tradeskill == false then
			has_tradeskill = true
		end
		
		if v == false then
			msg_status = "|cffff0000" .. L.failed .. "|r"
		else
			msg_status = "|cff00ff00" .. L.success .. "|r"
		end
		
		self:Print("    " .. k .. ": " .. msg_status)
	end
	
	if has_tradeskill == false then
		self:Print("    " .. L.none)
	end
end


function LZProfiler:PurgeTradeskills()
	LZRoot.Professions = {}
	LZParsedStatus = {}
	
	self:Print(L.purgedTradeskills)
end

function LZProfiler:PurgeAll()
	LZRoot.Professions = {}
	LZParsedStatus = {}
	
	local realm_name = GetRealmName()
	
	for k,_ in pairs(LZProfile.Characters[realm_name]) do
		if k ~= UnitName("player") then
			LZProfile.Characters[realm_name][k] = nil
		end
	end
	
	self:Print(L.purgedAll)
end